/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package pod

import (
	corev1 "k8s.io/api/core/v1"
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
	container "k8s.libre.sh/objects/container"
)

// +kubebuilder:object:generate=true
type Pod struct {
	*PodSpec `json:",inline"`
	//	Parameters parameters.Parameters `json:"parameters,omitempty"`
}

// +kubebuilder:object:generate=true
type PodSpec struct {
	*meta.ObjectMeta    `json:"meta,omitempty"`
	Resources           corev1.ResourceRequirements `json:"resources,omitempty" protobuf:"bytes,8,opt,name=resources"`
	SecurityContext     *corev1.PodSecurityContext  `json:"securityContext,omitempty" protobuf:"bytes,14,opt,name=securityContext"`
	Affinity            *corev1.Affinity            `json:"affinity,omitempty"`
	RestartPolicy       corev1.RestartPolicy        `json:"restartPolicy,omitempty"`
	Tolerations         []corev1.Toleration         `json:"tolerations,omitempty"`
	NodeSelector        map[string]string           `json:"nodeSelector,omitempty"`
	container.Container `json:",inline"`
	ExtraContainers     []container.Container `json:"extraContainers,omitempty"`
	InitContainers      []container.Container `json:"initContainers,omitempty"`
}

func (r *PodSpec) GetPodAffinity() *corev1.Affinity                  { return r.Affinity }
func (r *PodSpec) GetPodRestartPolicy() corev1.RestartPolicy         { return r.RestartPolicy }
func (r *PodSpec) GetPodNodeSelector() map[string]string             { return r.NodeSelector }
func (r *PodSpec) GetPodTolerations() []corev1.Toleration            { return r.Tolerations }
func (r *PodSpec) GetPodSecurityContext() *corev1.PodSecurityContext { return r.SecurityContext }

func (o *PodSpec) Mutate(obj interfaces.Object) error {
	return nil
}

func (spec *PodSpec) GetContainers() []container.Mutate {
	if len(spec.Container.TemplateSpec.Name) == 0 {
		spec.Container.TemplateSpec.Name = spec.ObjectMeta.GetComponent()
	}
	containers := []container.Mutate{}

	containers = append(containers, &spec.Container)
	return containers

}
